using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using GBPVR.Public;

namespace gbweb
{
	/// <summary>
	/// Summary description for Credits.
	/// </summary>
	public class Credits : System.Web.UI.Page
	{

		private Settings guideParams;
		protected System.Web.UI.WebControls.Table CreditsTable;
		protected System.Web.UI.WebControls.Label showChannel;
		protected System.Web.UI.WebControls.Label showTimes;
		protected System.Web.UI.WebControls.Label showStartDate;
		protected System.Web.UI.WebControls.Label showSubTitle;
		protected System.Web.UI.WebControls.Label showTitle;
		protected System.Web.UI.WebControls.Table ShowDescTable;
		protected string programUniqueIdentifier;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			guideParams = Global.Settings;
			
			if (!IsPostBack) 
			{
				
				Schedule scheduleHelper = Global.Schedule;
				Programme programme = null;

				programme = scheduleHelper.GetProgrammeByOID(int.Parse(Request.Params["id"]));
				// show programme details
				showTitle.Text = programme.getTitle();
				programUniqueIdentifier = programme.getUniqueProgrammeIdentifier();
				showSubTitle.Text = programme.getSubTitle();

				string description = programme.getDescription();

				//Check to see if the user uses the ZapTools Extended database
				bool zapToolsEnabled = ZapTools.Initialize();
				if (zapToolsEnabled)
				{
					//Open the ZapTools DB
					ZapTools.OpenZapDB();
				}

				//Check to see if the user uses the Zap2it EPG...if so load the genre and rating from the native gbpvr database
				if (ZapTools.checkZapEPG())
				{
					//Add a line break if there is a description to provide seperation space
					if (description.Length > 0)
					{
						description = description + "<br>";
					}

					//Add the show Genre to the line
					if (zapToolsEnabled)
					{
						string genreCheck = ZapTools.GetGenre(programme.getUniqueProgrammeIdentifier());
						if (genreCheck != "Unknown" && genreCheck != "")
						{
						description = description + "<br>" + "Type : " + genreCheck;
						}
					}

					//Retrieve the rating for the show
					string rating = programme.getRating();
					if (rating != null && rating != "")
					{
						description = description + "<br>" + "  Rated: " + programme.getRating();
					}
				}

				//If the user has ZapTools append additional information to the description when found
				if (zapToolsEnabled)
				{
					//Check the ZapTools supplemental database to see if the show is a repeat
					if (ZapTools.IsRepeat(programme.getUniqueProgrammeIdentifier()))
					{
						//If show is a repeat tag an indicator at the end of the description indicating so
						description = description + "<br><B>  (Repeat)</B>";
					}
			
					//Pull all the cast/crew found for the show
					ArrayList cast = ZapTools.GetCast(programme.getUniqueProgrammeIdentifier());

					//Close the ZapTools DB
					ZapTools.CloseZapDB();

					//If any cast was found load up the information
					if (cast.Count > 0)
					{
						//Create the table arrays to hold any found crew
						ArrayList Actors		= new ArrayList();
						ArrayList Directors		= new ArrayList();
						ArrayList Producers		= new ArrayList();
						ArrayList Writers		= new ArrayList();
						ArrayList ExecProducers = new ArrayList();
						ArrayList GuestStars	= new ArrayList();
						ArrayList Hosts			= new ArrayList();

						foreach (ZapTools.Crew crew in cast)
						{
							switch (crew.Role)
							{
								case ZapTools.Role.Actor:
									Actors.Add(crew);
									break;
								case ZapTools.Role.Director:
									Directors.Add(crew);
									break;
								case ZapTools.Role.Producer:
									Producers.Add(crew);
									break;
								case ZapTools.Role.Writer:
									Writers.Add(crew);
									break;
								case ZapTools.Role.ExecProducer:
									ExecProducers.Add(crew);
									break;
								case ZapTools.Role.GuestStar:
									GuestStars.Add(crew);
									break;
								case ZapTools.Role.Host:
									Hosts.Add(crew);
									break;
							}
						}

						//Create first table header row
						TableRow headerRow = new TableRow();

						//Load the header row with the titles
						TableCell actors = new TableCell();
						actors.Text = "<b>Actors</b>";
						actors.CssClass = "header";
						actors.Width = 75;
						actors.Wrap = false;
						headerRow.Cells.Add(actors);

						TableCell directors = new TableCell();
						directors.Text = "<b>Directors</b>";
						directors.CssClass = "header";
						directors.Width = 75;
						directors.Wrap = false;
						headerRow.Cells.Add(directors);

						TableCell producers = new TableCell();
						producers.Text = "<b>Exec. Producers</b>";
						producers.CssClass = "header";
						producers.Width = 75;
						producers.Wrap = false;
						headerRow.Cells.Add(producers);

						TableCell writers = new TableCell();
						writers.Text = "<b>Producers</b>";
						writers.CssClass = "header";
						writers.Width = 75;
						writers.Wrap = false;
						headerRow.Cells.Add(writers);

						TableCell execproducers = new TableCell();
						execproducers.Text = "<b>Writers</b>";
						execproducers.CssClass = "header";
						execproducers.Width = 75;
						execproducers.Wrap = false;
						headerRow.Cells.Add(execproducers);

						TableCell gueststars = new TableCell();
						gueststars.Text = "<b>Guest Stars</b>";
						gueststars.CssClass = "header";
						gueststars.Width = 75;
						gueststars.Wrap = false;
						headerRow.Cells.Add(gueststars);

						TableCell hosts = new TableCell();
						hosts.Text = "<b>Hosts</b>";
						hosts.CssClass = "header";
						hosts.Width = 75;
						hosts.Wrap = false;
						headerRow.Cells.Add(hosts);

						CreditsTable.Rows.Add(headerRow);
					
						//Load the rows with data
						while (
							(Actors.Count			> 0) || 
							(Directors.Count		> 0) || 	
							(Producers.Count		> 0) || 	
							(Writers.Count			> 0) || 
							(ExecProducers.Count	> 0) || 
							(GuestStars.Count		> 0) || 
							(Hosts.Count			> 0))
						{
							TableRow crewRow = new TableRow();
							AddCell(crewRow, Actors			);
							AddCell(crewRow, Directors		);
							AddCell(crewRow, ExecProducers	);
							AddCell(crewRow, Producers		);
							AddCell(crewRow, Writers		);
							AddCell(crewRow, GuestStars		);
							AddCell(crewRow, Hosts			);
							CreditsTable.Rows.Add(crewRow);
						}
					}
					else
					{
						//Load the header row with the titles
						//Create first table header row
						TableRow headerRow = new TableRow();
						TableCell noactors = new TableCell();

						noactors.Text = "<b>No Credits Available</b>";
						noactors.CssClass = "header";
						noactors.Width = 75;
						noactors.Wrap = false;
						headerRow.Cells.Add(noactors);
						CreditsTable.Rows.Add(headerRow);
					}
				}

				
				//Load the show description/date/channel
				TableCell desc = new TableCell();
				desc.Text = description;
				TableRow descRow = new TableRow();
				descRow.Cells.Add(desc);
				ShowDescTable.Rows.Add(descRow);

				showStartDate.Text = programme.getStartTime().ToLongDateString();
				showTimes.Text = programme.getStartTime().ToShortTimeString() + " - " + programme.getEndTime().ToShortTimeString();

				Channel channel = scheduleHelper.GetChannelByOID(programme.getChannelOID());
				string channelName = channel.getName();
				if (channelName.StartsWith(channel.getChannelNumber().ToString())) // cosmetic stuff for my american friends
				{
					channelName = channelName.Substring(channel.getChannelNumber().ToString().Length);
				}
				
				showChannel.Text = "<span class=\"channelnumber\"><b>" + channel.getChannelNumber() + "</b></span> (<span class=\"channelname\"><nobr>" + channelName + "</nobr></span>)";

			}
		}

		private void AddCell(TableRow crewRow, ArrayList crewlist)
		{
			if (crewlist.Count > 0)
			{
				ZapTools.Crew crew = (ZapTools.Crew)crewlist[0];
				crewlist.Remove(crew);
				TableCell cell = new TableCell();
				cell.Text = "<a href=\"http://www.imdb.com/Find?for=" + crew.GivenName + " " + crew.SurName + "\" onclick=\"EditPop2(this,'IMDB');return false;\">" + crew.GivenName + " " + crew.SurName + "</a>";
				crewRow.Cells.Add(cell);
			}
			else
			{
				crewRow.Cells.Add(new TableCell());
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
