using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using GBPVR.Public;

namespace gbweb
{
	/// <summary>
	/// Summary description for Details.
	/// </summary>
	public class Details : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label showTitle;
		protected System.Web.UI.WebControls.Label showSubTitle;
		protected System.Web.UI.WebControls.Panel subTitlePanel;
		protected System.Web.UI.WebControls.Panel descriptionPanel;
		protected System.Web.UI.WebControls.Panel recordingOptionsPanel;
		protected System.Web.UI.WebControls.Panel recordingDetailsPanel;
		protected System.Web.UI.WebControls.Label showStartDate;
		protected System.Web.UI.WebControls.Label showTimes;
		protected System.Web.UI.WebControls.Label showChannel;
		protected System.Web.UI.WebControls.Label showDescription;
		protected System.Web.UI.WebControls.Label recordingQuality;
		protected System.Web.UI.HtmlControls.HtmlGenericControl divUniqueID;
		protected System.Web.UI.HtmlControls.HtmlAnchor watchLink;
		protected System.Web.UI.HtmlControls.HtmlGenericControl spanRecordCancel;
		protected System.Web.UI.WebControls.DropDownList quality;

		private Settings guideParams;
		protected System.Web.UI.WebControls.LinkButton lnkRecordCancel;
		protected System.Web.UI.WebControls.RadioButtonList radioTime;
		protected System.Web.UI.WebControls.RadioButtonList radioDay;
		protected System.Web.UI.WebControls.CheckBoxList checkedDays;
		protected System.Web.UI.WebControls.TextBox keepRecordings;
		protected System.Web.UI.WebControls.Label RecordMessage;
		protected string programUniqueIdentifier;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
			guideParams = Global.Settings;
			
			if (!IsPostBack) 
			{
				Schedule scheduleHelper = Global.Schedule;
				Programme programme = null;

				// if this show was already scheduled, view=cancel would have been passed in
				if (Request.Params["rid"] != null)
				{				
					recordingOptionsPanel.Visible = false;
					spanRecordCancel.InnerText = "Recording Not Found!";
					lnkRecordCancel.CommandName = "cancel";

					ScheduledRecording scheduledRecording = scheduleHelper.GetScheduledRecordingByOID(Convert.ToInt32(Request.Params["rid"]));

					if (scheduledRecording != null)
					{
						watchLink.HRef = Download.GetDownloadUrl(false, false, Convert.ToInt32(Request.Params["rid"]));
						watchLink.Visible = false;
						// reload recording list
						string recorded_quality;
						switch (scheduledRecording.getQualitySetting())
						{
							case (int)Schedule.Quality.Low:
								recorded_quality = "Low";
								break;
							case (int)Schedule.Quality.Medium:
								recorded_quality = "Medium";
								break;
							case (int)Schedule.Quality.High:
								recorded_quality = "High";
								break;
							case (int)Schedule.Quality.Custom1:
								recorded_quality = "Custom 1";
								break;
							case (int)Schedule.Quality.Custom2:
								recorded_quality = "Custom 2";
								break;
							case (int)Schedule.Quality.LiveTV:
								recorded_quality = "LiveTV";
								break;
							default:
								recorded_quality = string.Empty;
								break;
						}
						recordingQuality.Text = recorded_quality;
						switch (scheduledRecording.getRecordingStatus())
						{
							case ScheduledRecording.STATUS_PENDING:
								lnkRecordCancel.CssClass = "btn-normal-large";
								spanRecordCancel.InnerText = "Cancel Recording";
								break;
							case ScheduledRecording.STATUS_IN_PROGRESS:
								lnkRecordCancel.CssClass = "btn-normal-large-red";
								spanRecordCancel.InnerText = "Cancel Recording";
								watchLink.Visible = true;
								break;
							case ScheduledRecording.STATUS_COMPLETED:
								lnkRecordCancel.CssClass = "btn-normal-large-red";
								spanRecordCancel.InnerText = "Delete Recording";
								watchLink.Visible = true;
								break;
							case ScheduledRecording.STATUS_COMPLETED_WITH_ERROR:
								lnkRecordCancel.CssClass = "btn-normal-large-red";
								spanRecordCancel.InnerText = "Delete Recording";
								break;
							case ScheduledRecording.STATUS_CONFLICT:
								lnkRecordCancel.CssClass = "btn-normal-large";
								spanRecordCancel.InnerText = "Cancel Recording";
								break;
							case ScheduledRecording.STATUS_PLACE_HOLDER:
								lnkRecordCancel.CssClass = "btn-normal-large";
								spanRecordCancel.InnerText = "Cancel Recording";
								break;
							case ScheduledRecording.STATUS_DELETED:
								lnkRecordCancel.CssClass = "btn-normal-large";
								spanRecordCancel.InnerText = "Record This Episode";
								break;
							default:
								lnkRecordCancel.CssClass = "btn-normal-large";
								spanRecordCancel.InnerText = "Unknown Status!";
								break;
						}
					}

					programme = scheduleHelper.GetProgrammeByOID(int.Parse(Request.Params["id"]));
				}
				else
				{
					recordingDetailsPanel.Visible = false;
					spanRecordCancel.InnerText = "Record This Episode";
					lnkRecordCancel.CommandName = "record";
					programme = scheduleHelper.GetProgrammeByOID(int.Parse(Request.Params["id"]));
				}

				// show programme details
				showTitle.Text = programme.getTitle();
				programUniqueIdentifier = programme.getUniqueProgrammeIdentifier();
				showSubTitle.Text = programme.getSubTitle();

				string description = programme.getDescription();
				
				//Check to see if the user uses the ZapTools Extended database
				bool zapToolsEnabled = ZapTools.Initialize();
				if (zapToolsEnabled)
				{
					//Open the ZapTools DB
					ZapTools.OpenZapDB();
				}

				//Check to see if the user uses the Zap2it EPG...if so load the genre and rating from the native gbpvr database
				if (ZapTools.checkZapEPG())
				{
					//Add a line break if there is a description to provide seperation space
					if (description.Length > 0)
					{
						description = description + "<br>";
					}

					//Retrieve the rating for the show
					string rating = programme.getRating();
					if (rating != null && rating != "")
					{
						description = description + "<br>" + "  Rated: " + programme.getRating();
					}
				}
				
				//Add the show Genre to the line
				//set the label for the list of genre
				ArrayList genreCheck = programme.getGenreList();
				for (int unknownPos = genreCheck.BinarySearch("unknown", new CaseInsensitiveComparer()); unknownPos > -1;
					unknownPos = genreCheck.BinarySearch("unknown", new CaseInsensitiveComparer()))
				{
					genreCheck.RemoveAt(unknownPos);
				}
				for (int unknownPos = genreCheck.BinarySearch(string.Empty, new CaseInsensitiveComparer()); unknownPos > -1;
					unknownPos = genreCheck.BinarySearch(string.Empty, new CaseInsensitiveComparer()))
				{
					genreCheck.RemoveAt(unknownPos);
				}
				if (genreCheck.Count > 0)
				{
					description += "<br/>Genre" + (genreCheck.Count > 1 ? "s" : string.Empty) + ": " + string.Join(", ", (string[])genreCheck.ToArray(typeof(string)));
				}

				// If the user has ZapTools append additional information to the description when found
				if (zapToolsEnabled)
				{
					// Pull the Airdate of the Show or the Release Year of the movie and the Start Rating for Movie
					string[] programInfo = ZapTools.GetProgramInfo(programUniqueIdentifier);

					if (programInfo.Length > 0)
					{
						if (programUniqueIdentifier.StartsWith("MV"))
						{
							if (programInfo[0] != "0")
							{
								description += "<br> Movie Release Year: " + programInfo[0];
							}
							if (programInfo[2] != "none")
							{
								description += "<br> Star Rating: " + programInfo[2];
							}
						}
						else
						{
							if (programInfo[1] != "none")
							{
								description += "<br> Original Air Date: " + programInfo[1];
							}
						}
					}
					
					//Check the ZapTools supplemental database to see if the show is a repeat
					if (ZapTools.IsRepeat(programUniqueIdentifier))
					{
						//If show is a repeat tag an indicator at the end of the description indicating so
						description += "<br><B>  (Repeat)</B>";
					}
					
					//Close the ZapTools DB
					ZapTools.CloseZapDB();
				}

				showDescription.Text = description;
				
				subTitlePanel.Visible = showSubTitle.Text.Length > 0;
				descriptionPanel.Visible = showDescription.Text.Length > 0;
				divUniqueID.Visible = programUniqueIdentifier.Length > 0;

				showStartDate.Text = programme.getStartTime().ToLongDateString();
				showTimes.Text = programme.getStartTime().ToShortTimeString() + " - " + programme.getEndTime().ToShortTimeString();

				Channel channel = scheduleHelper.GetChannelByOID(programme.getChannelOID());
				string channelName = channel.getName();
				if (channelName.StartsWith(channel.getChannelNumber().ToString())) // cosmetic stuff for my american friends
				{
					channelName = channelName.Substring(channel.getChannelNumber().ToString().Length);
				}
				showChannel.Text = "<span class=\"channelnumber\"><b>" + channel.getChannelNumber() + "</b></span> (<span class=\"channelname\"><nobr>" + channelName + "</nobr></span>)";

				//Set the default recording Quality
				quality.SelectedValue = guideParams.recordingQuality;
			}
		}

		private void lnkRecordCancel_Click(object sender, System.EventArgs e)
		{
			Schedule scheduleHelper = Global.Schedule;
			Programme programme = scheduleHelper.GetProgrammeByOID(int.Parse(Request.Params["id"]));
			bool scheduleReturn = true;
			if (lnkRecordCancel.CommandName == "record")
			{
				Schedule.Quality quality = 0;
				switch (Request.Params["quality"].ToLower())
				{
					case "high":
						quality = Schedule.Quality.High;
						break;

					case "medium":
						quality = Schedule.Quality.Medium;
						break;

					case "low":
						quality = Schedule.Quality.Low;
						break;

					case "custom1":
						quality = Schedule.Quality.Custom1;
						break;

					case "custom2":
						quality = Schedule.Quality.Custom2;
						break;
				}

				if (radioTime.SelectedValue == "once")		
				{
					// schedule one-off recording
					Schedule myschedule = Global.Schedule;
					scheduleReturn = myschedule.ScheduleOnce(programme, quality);
					if (scheduleReturn)
					{
						// build up a list of the recording we already know about
						IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
						if (!knownRecordings.Contains(programme.getOID()))
						{
							scheduleReturn = false;
						}
					}
				}
				else
				{
					//Schedule Season Recordings
					//Set the max number of recordings to keep for the show
					int keepnumRecordings	 = ReoccuringRecordingExtras.KEEP_ALL_FILES;
					if (keepRecordings.Text.ToString() != null)
					{
						try
						{
							keepnumRecordings	 = System.Convert.ToInt32(keepRecordings.Text);
						}
						catch
						{
							keepnumRecordings	 = ReoccuringRecordingExtras.KEEP_ALL_FILES;
						}
					}
					else
					{
						keepnumRecordings	 = ReoccuringRecordingExtras.KEEP_ALL_FILES;
					}
					
					Schedule myschedule = Global.Schedule;

					if (radioTime.SelectedValue == "thisTime")
					{
						// schedule season recording for any day and this time
						if (radioDay.SelectedValue == "anyDay")
						{
							scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleAnyDay, keepnumRecordings	);
						}
						// schedule season recording for only this day and this time
						else if (radioDay.SelectedValue == "thisDay")
						{
							scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleThisDay, keepnumRecordings	);
							if (scheduleReturn)
							{
								// build up a list of the recording we already know about
								IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
								if (!knownRecordings.Contains(programme.getOID()))
								{
									scheduleReturn = false;
								}
							}
						}
						// schedule season recording for the selected days and this time
						else
						{
							ArrayList dayList = new ArrayList();
							// Iterate through the Items collection of the checkedDays 
							// control and create the correct number for the season recording.
							for (int i=0; i<checkedDays.Items.Count; i++)
							{
								if (checkedDays.Items[i].Selected)
								{
									switch (checkedDays.Items[i].Text)
									{
										case "Monday":
											dayList.Add(Schedule.Day.Monday);
											break;
										case "Tuesday":
											dayList.Add(Schedule.Day.Tuesday);
											break;
										case "Wednesday":
											dayList.Add(Schedule.Day.Wednesday);
											break;
										case "Thursday":
											dayList.Add(Schedule.Day.Thursday);
											break;
										case "Friday":
											dayList.Add(Schedule.Day.Friday);
											break;
										case "Saturday":
											dayList.Add(Schedule.Day.Saturday);
											break;
										case "Sunday":
											dayList.Add(Schedule.Day.Sunday);
											break;
									}
								}
							}
							scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleSpecificDays, keepnumRecordings	, dayList);
							if (scheduleReturn)
							{
								// build up a list of the recording we already know about
								IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
								if (!knownRecordings.Contains(programme.getOID()))
								{
									scheduleReturn = false;
								}
							}
						}
					}
					else
					{
						// schedule season recording for anyday anytime
						if (radioDay.SelectedValue == "anyDay")
						{
							scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleAnyDay, keepnumRecordings	);
							if (scheduleReturn)
							{
								// build up a list of the recording we already know about
								IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
								if (!knownRecordings.Contains(programme.getOID()))
								{
									scheduleReturn = false;
								}
							}
						}
							// schedule season recording for only this day and thisany time
						else if (radioDay.SelectedValue == "thisDay")
						{
							scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleThisDay, keepnumRecordings	);
							if (scheduleReturn)
							{
								// build up a list of the recording we already know about
								IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
								if (!knownRecordings.Contains(programme.getOID()))
								{
									scheduleReturn = false;
								}
							}
						}
							// schedule season recording for the selected days and this time
						else
						{
							ArrayList dayList = new ArrayList();
							// Iterate through the Items collection of the checkedDays 
							// control and create the correct number for the season recording.
							for (int i=0; i<checkedDays.Items.Count; i++)
							{
								if (checkedDays.Items[i].Selected)
								{
									switch (checkedDays.Items[i].Text)
									{
										case "Monday":
											dayList.Add(Schedule.Day.Monday);
											break;
										case "Tuesday":
											dayList.Add(Schedule.Day.Tuesday);
											break;
										case "Wednesday":
											dayList.Add(Schedule.Day.Wednesday);
											break;
										case "Thursday":
											dayList.Add(Schedule.Day.Thursday);
											break;
										case "Friday":
											dayList.Add(Schedule.Day.Friday);
											break;
										case "Saturday":
											dayList.Add(Schedule.Day.Saturday);
											break;
										case "Sunday":
											dayList.Add(Schedule.Day.Sunday);
											break;
									}
								}
							}
							scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleSpecificDays, keepnumRecordings	, dayList);
							if (scheduleReturn)
							{
								// build up a list of the recording we already know about
								IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();
								if (!knownRecordings.Contains(programme.getOID()))
								{
									scheduleReturn = false;
								}
							}
						}
					}
				}

				if (scheduleReturn)
				{
					// close popup details window
					Page.RegisterStartupScript("startupScript", "<script language=JavaScript>reloadAndClose();</script>");
				}
				else
				{
					RecordMessage.Text = "Recording Failed!";
					RecordMessage.Visible = true;
				}
			}
			else if (lnkRecordCancel.CommandName == "cancel")
			{
				ScheduledRecording scheduledRecording = scheduleHelper.GetScheduledRecordingByOID(Convert.ToInt32(Request.Params["rid"]));
				if (scheduledRecording != null)
				{
					scheduleHelper.CancelScheduledRecording(scheduledRecording);
				}
				Page.RegisterStartupScript("startupScript", "<script language=JavaScript>reloadAndClose();</script>");
			}
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.lnkRecordCancel.Click += new System.EventHandler(this.lnkRecordCancel_Click);
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion

	}
}
