using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using GBPVR.Public;

namespace gbweb
{
	/// <summary>
	/// Summary description for Library.
	/// </summary>
	public class Library : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Table libraryView;
		protected System.Web.UI.HtmlControls.HtmlTableCell coverArt;
		protected System.Web.UI.HtmlControls.HtmlAnchor playAlbum;
		protected System.Web.UI.HtmlControls.HtmlAnchor downloadAlbum;
		protected Header header;

		protected string Mode;
		protected string DisplayPath;

		public static string GetRealPath(string vpath)
		{
			string[] pathparts = vpath.Split(new char[] {'/'}, 3);
			string Mode = CultureInfo.CurrentUICulture.TextInfo.ToTitleCase(pathparts[0]);

			Hashtable directories = (Hashtable)HttpContext.Current.Session[Mode + "LibraryDirectories"];
			if (directories == null)
			{

				string directorylist = "Video~|Music~";
				XmlNode node = Global.Config.SelectSingleNode("/settings/" + Mode + "LibraryDirectory");
				if (node != null)
				{
					directorylist = node.InnerText;
				}

				string[] directorymappings = directorylist.Split('|');
				directories = new Hashtable();
				foreach (string directorymapping in directorymappings)
				{
					string[] mapping = directorymapping.Split('~');
					if ((directorymapping.Length > 0) && (mapping.Length > 1))
						directories[mapping[0]] = mapping[1];
				}
				HttpContext.Current.Session[Mode + "LibraryDirectories"] = directories;
			}

			string path = null;
			if (pathparts.Length == 2)
			{
				path = (string)directories[pathparts[1]];
			}
			else if (pathparts.Length == 3)
			{
				path = Path.Combine((string)directories[pathparts[1]], pathparts[2].Replace('/', Path.DirectorySeparatorChar));
			}

			return path;
		}

		public static Regex GetRegex(string mode)
		{
			string filterRegex;
			switch (mode.ToLower())
			{
				case "video":
					filterRegex = Global.Settings.videoFiles;
					break;
				case "music":
					filterRegex = Global.Settings.musicFiles;
					break;
				default:
					filterRegex = string.Empty;
					break;
			}
			return new Regex(filterRegex, RegexOptions.Compiled | RegexOptions.IgnoreCase);
		}

		private void Page_Load(object sender, System.EventArgs e)
		{
			playAlbum.Visible = false;
			
			string vpath = string.Empty;
			if ((Request["path"] != null) && (Request["path"].Length != 0))
			{
				vpath = Request["path"];
			}
			string[] pathparts = vpath.Split(new char[] {'/'}, 3);
			Mode = CultureInfo.CurrentUICulture.TextInfo.ToTitleCase(pathparts[0]);

			Regex fileFilter = GetRegex(Mode);

			header.ActiveTab = Mode.ToLower() + "library";

			string path = GetRealPath(vpath);

			ArrayList folders = new ArrayList();
			ArrayList files = new ArrayList();

			if (pathparts.Length == 1)
			{
				Hashtable directories = (Hashtable)HttpContext.Current.Session[Mode + "LibraryDirectories"];
				ArrayList keys = new ArrayList(directories.Keys);
				keys.Sort();
				foreach (string name in keys)
					folders.Add(name);
			}
			else
			{
				if (pathparts.Length == 2)
					DisplayPath = " - " + pathparts[1];
				else if (pathparts.Length == 3) 
				{
					DisplayPath = " - " + pathparts[1] + "/" + pathparts[2];
				}

				try
				{
					folders.Add("..");

					string[] dirs = Directory.GetDirectories(path);
					Array.Sort(dirs);
					foreach (string dir in dirs)
					{
						if (dir == Path.GetFullPath(dir))
							folders.Add(Path.GetFileName(dir));
					}

					string[] filenames = Directory.GetFiles(path);
					Array.Sort(filenames);
					foreach (string filename in filenames)
					{
						if (filename == Path.GetFullPath(filename))
							files.Add(Path.GetFileName(filename));
					}
				}
				catch (Exception ex)
				{
					Logger.Error(ex.ToString());
				}
			}

			string root = string.Join("/", pathparts);
			if (root.Length > 0) root += "/";

			TableCell colShow = new TableCell();
			TableRow recordingRow;
			foreach (string folder in folders)
			{

				recordingRow = new TableRow();
				recordingRow.CssClass = "tbg";

				colShow = new TableCell();
				colShow.VerticalAlign = VerticalAlign.Top;
				colShow.CssClass = "folder";

				if (folder != "..")
				{
					path = root + Server.UrlEncode(folder);
				}
				else 
				{
					path = string.Join("/", pathparts);
					path = path.Substring(0, path.LastIndexOf("/"));
				}

				colShow.Text = "<a href=\"?path=" + path + "\" >[" + folder + "]</a>";

				recordingRow.Cells.Add(colShow);

				libraryView.Rows.Add(recordingRow);

			}

			Regex coverFilter = new Regex(Global.Settings.coverFiles, RegexOptions.Compiled | RegexOptions.IgnoreCase);
			string coverImage = null;
			foreach (string file in files)
			{
				if (fileFilter.IsMatch(file))
				{
					playAlbum.Visible = true;
					
					recordingRow = new TableRow();
					recordingRow.CssClass = "tbg";

					colShow = new TableCell();
					colShow.VerticalAlign = VerticalAlign.Top;
					colShow.CssClass = "file";

					colShow.Text = 
						"<a href=\"" + Download.GetDownloadUrl(true, true, Download.InternalFiles.LibraryFile, root + file) + "\">[dl]</a> " + 
						"<a href=\"" + Download.GetDownloadUrl(false, false, Download.InternalFiles.LibraryFile, root + file) + "\">" + file + "</a> ";

					recordingRow.Cells.Add(colShow);

					libraryView.Rows.Add(recordingRow);
				}
				if ((coverFilter.IsMatch(file) && (coverImage == null)))
				{
					coverImage = Download.GetDownloadUrl(false, true, Download.InternalFiles.LibraryFile, root + file);
				}
			}

			if (coverImage != null)
			{
				coverArt.Attributes.Add("background", coverImage);
			}

			downloadAlbum.Visible = playAlbum.Visible;
			if (playAlbum.Visible)
			{
				playAlbum.HRef = Download.GetDownloadUrl(false, false, Download.InternalFiles.LibraryFolder, root);
				downloadAlbum.HRef = Download.GetDownloadUrl(true, true, Download.InternalFiles.LibraryFolder, root);
			} 

		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}

}
