using System;
using System.Collections;
using System.Collections.Specialized;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using GBPVR.Public;

using gbweb.classes;

namespace gbweb
{
	/// <summary>
	/// Summary description for SearchResults.
	/// </summary>
	public class SearchResults : System.Web.UI.Page
	{
		protected System.Web.UI.HtmlControls.HtmlForm Form1;
		protected System.Web.UI.WebControls.LinkButton hlRecord;
		protected System.Web.UI.WebControls.CheckBoxList checkedDays;
		protected System.Web.UI.WebControls.DropDownList DropDownList1;
		protected System.Web.UI.WebControls.RadioButtonList radioDay;
		protected System.Web.UI.WebControls.RadioButtonList radioTime;
		protected System.Web.UI.WebControls.DropDownList quality;
		protected System.Web.UI.WebControls.Table tableResults;
		protected System.Web.UI.WebControls.TextBox keepRecordings;
		protected System.Web.UI.WebControls.Label RecordMessage;
		protected System.Web.UI.WebControls.DropDownList Recordings;

		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!IsPostBack)
			{
				if ((Request.Params["checkTitle"] != null) || (Request.Params["checkDesc"] != null) || (Request.Params["checkSubtitle"] != null) || (Request.Params["listChannels"] != null) ||
					(Request.Params["matchTitle"] != null) || (Request.Params["matchDesc"] != null) || (Request.Params["matchSubtitle"] != null) || (Request.Params["matchUniqueID"] != null)  ||
					(Request.Params["textMinLength"] != null) && (Request.Params["textMinLength"].Length > 0) ||
					((Request.Params["textMaxLength"] != null) && (Request.Params["textMaxLength"].Length > 0)))
				{
					int minLength = int.MinValue;
					int maxLength = int.MaxValue;
					if ((Request.Params["textMinLength"] != null) && (Request.Params["textMinLength"].Length > 0)) minLength = int.Parse(Request.Params["textMinLength"]);
					if ((Request.Params["textMaxLength"] != null) && (Request.Params["textMaxLength"].Length > 0)) maxLength = int.Parse(Request.Params["textMaxLength"]);
					string[] channels = new string[0] {};
					if (Request.Params["listChannels"] != null) channels = Request.Params["listChannels"].Split(',');

					ViewState["checkTitle"] = Request.Params["checkTitle"] != null;
					ViewState["checkDesc"] = Request.Params["checkDesc"] != null;
					ViewState["checkSubtitle"] = Request.Params["checkSubtitle"] != null;
					ViewState["matchTitle"] = Request.Params["matchTitle"] != null;
					ViewState["matchDesc"] = Request.Params["matchDesc"] != null;
					ViewState["matchSubtitle"] = Request.Params["matchSubtitle"] != null;
					ViewState["matchUniqueID"] = Request.Params["matchUniqueID"] != null;
					ViewState["textKeyWord"] = Request.Params["textKeyWord"];
					ViewState["genreList"] = Request.Params["genreList"];
					ViewState["minLength"] = minLength;
					ViewState["maxLength"] = maxLength;
					ViewState["channels"] = channels;

					//Set the default recording Quality
					quality.SelectedValue = Global.Settings.recordingQuality;
				}
				else
				{
					Response.Redirect("Search.aspx");
				}
			}
		}

		private void SearchResults_PreRender(object sender, System.EventArgs e)
		{
			SearchForEpisode(
				(bool)ViewState["checkTitle"],
				(bool)ViewState["checkDesc"],
				(bool)ViewState["checkSubtitle"],
				(bool)ViewState["matchTitle"],
				(bool)ViewState["matchDesc"],
				(bool)ViewState["matchSubtitle"],
				(bool)ViewState["matchUniqueID"],
				(string)ViewState["textKeyWord"],
				(int)ViewState["minLength"],
				(int)ViewState["maxLength"],
				(string[])ViewState["channels"],
				(string)ViewState["genreList"]);
		}

		private void hlRecord_Click(object sender, System.EventArgs e)
		{
			// schedule recordings
			Schedule.Quality quality = 0;
			switch (Request.Params["quality"].ToLower())
			{
				case "high":
					quality = Schedule.Quality.High;
					break;

				case "medium":
					quality = Schedule.Quality.Medium;
					break;

				case "low":
					quality = Schedule.Quality.Low;
					break;

				case "custom1":
					quality = Schedule.Quality.Custom1;
					break;

				case "custom2":
					quality = Schedule.Quality.Custom2;
					break;
			}

			Schedule scheduleHelper = Global.Schedule;
			if (radioTime.SelectedValue == "once")
			{
				foreach (string param in Request.Params.Keys)
				{
					if (param.StartsWith("item"))
					{
						Programme programme = scheduleHelper.GetProgrammeByOID(int.Parse(param.Substring(4)));
						Schedule myschedule = Global.Schedule;
						bool scheduleReturn = myschedule.ScheduleOnce(programme, quality);
						Channel chnl = myschedule.GetChannelByOID(programme.getChannelOID());
						Recordings.Items.Add(programme.getOID().ToString()+ "@" + chnl.channelName + " " + programme.getTitle().ToString() + " " + programme.getStartTime().ToLongDateString() + " " + programme.getStartTime().ToShortTimeString() + " - " + programme.getEndTime().ToShortTimeString());
					}
				}
			}
			else
			{
				HybridDictionary processedRecordings = new HybridDictionary();
				foreach (string param in Request.Params.Keys)
				{
					if (param.StartsWith("item"))
					{
						Programme programme = scheduleHelper.GetProgrammeByOID(int.Parse(param.Substring(4)));

						string uniqueProgrammeKey = programme.getChannelOID().ToString() + ":" + programme.getTitle();

						if (!processedRecordings.Contains(uniqueProgrammeKey))
						{
							//Set the max number of recordings to keep for the show
							int keepnumRecordings = ReoccuringRecordingExtras.KEEP_ALL_FILES;
							if (keepRecordings.Text.ToString() != null)
							{
								try
								{
									keepnumRecordings = Convert.ToInt32(keepRecordings.Text);
								}
								catch
								{
									keepnumRecordings = ReoccuringRecordingExtras.KEEP_ALL_FILES;
								}
							}
							else
							{
								keepnumRecordings = ReoccuringRecordingExtras.KEEP_ALL_FILES;
							}

							Schedule myschedule = Global.Schedule;

							if (radioTime.SelectedValue == "thisTime")
							{
								// schedule season recording for any day and this time
								if (radioDay.SelectedValue == "anyDay")
								{
									bool scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleAnyDay, keepnumRecordings	);
								}
									// schedule season recording for only this day and this time
								else if (radioDay.SelectedValue == "thisDay")
								{
									bool scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleThisDay, keepnumRecordings	);
								}
									// schedule season recording for the selected days and this time
								else
								{
									ArrayList dayList = new ArrayList();
									// Iterate through the Items collection of the checkedDays 
									// control and create the correct number for the season recording.
									for (int i=0; i<checkedDays.Items.Count; i++)
									{
										if (checkedDays.Items[i].Selected)
										{
											switch (checkedDays.Items[i].Text)
											{
												case "Monday":
													dayList.Add(Schedule.Day.Monday);
													break;
												case "Tuesday":
													dayList.Add(Schedule.Day.Tuesday);
													break;
												case "Wednesday":
													dayList.Add(Schedule.Day.Wednesday);
													break;
												case "Thursday":
													dayList.Add(Schedule.Day.Thursday);
													break;
												case "Friday":
													dayList.Add(Schedule.Day.Friday);
													break;
												case "Saturday":
													dayList.Add(Schedule.Day.Saturday);
													break;
												case "Sunday":
													dayList.Add(Schedule.Day.Sunday);
													break;
											}
										}
									}
									bool scheduleReturn = myschedule.ScheduleThisTime(programme, quality, Schedule.DayType.SheduleSpecificDays, keepnumRecordings	, dayList);
								}
							}
							else
							{
								// schedule season recording for anyday anytime
								if (radioDay.SelectedValue == "anyDay")
								{
									bool scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleAnyDay, keepnumRecordings	);
								}
									// schedule season recording for only this day and thisany time
								else if (radioDay.SelectedValue == "thisDay")
								{
									bool scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleThisDay, keepnumRecordings	);
								}
									// schedule season recording for the selected days and this time
								else
								{
									ArrayList dayList = new ArrayList();
									// Iterate through the Items collection of the checkedDays 
									// control and create the correct number for the season recording.
									for (int i=0; i<checkedDays.Items.Count; i++)
									{
										if (checkedDays.Items[i].Selected)
										{
											switch (checkedDays.Items[i].Text)
											{
												case "Monday":
													dayList.Add(Schedule.Day.Monday);
													break;
												case "Tuesday":
													dayList.Add(Schedule.Day.Tuesday);
													break;
												case "Wednesday":
													dayList.Add(Schedule.Day.Wednesday);
													break;
												case "Thursday":
													dayList.Add(Schedule.Day.Thursday);
													break;
												case "Friday":
													dayList.Add(Schedule.Day.Friday);
													break;
												case "Saturday":
													dayList.Add(Schedule.Day.Saturday);
													break;
												case "Sunday":
													dayList.Add(Schedule.Day.Sunday);
													break;
											}
										}
									}
									bool scheduleReturn = myschedule.ScheduleAnyTime(programme, quality, Schedule.DayType.SheduleSpecificDays, keepnumRecordings	, dayList);
								}
							}

							// drop out after first season recording for the show
							// (it really doesn't matter what we store here, but myRecording sounds usefull)
							processedRecordings.Add(uniqueProgrammeKey, programme);
							Channel chnl = myschedule.GetChannelByOID(programme.getChannelOID());
							Recordings.Items.Add(programme.getOID().ToString()+ "@" + chnl.channelName + " " + programme.getTitle().ToString() + " " + programme.getStartTime().ToLongDateString() + " " + programme.getStartTime().ToShortTimeString() + " - " + programme.getEndTime().ToShortTimeString());
						}
					}
				}
			}
		}

		private void SearchForEpisode(
			bool checkTitle, bool checkDesc, bool checkSubtitle,
			bool matchTitle, bool matchDesc, bool matchSubtitle, bool matchUniqueID,
			String searchFor, int minLength, int maxLength,
			string[] channels, string selectedGenre)
		{
			Schedule scheduleHelper = Global.Schedule;

			ProgrammeDisplay displayProgramme = new ProgrammeDisplay();

			// build up a list of the recording we already know about
			IDictionary knownRecordings = scheduleHelper.LoadKnownRecordings();

			// Search for an episode listing, set the start time and the end time
			DateTime startTime = DateTime.Now;
			DateTime endTime = startTime.AddDays((double) 30);
			IList allChannels = scheduleHelper.GetListingsForTimePeriod(startTime, endTime);

			createHeaderRow();

			if (searchFor != null)
			{
				searchFor = searchFor.ToUpper();
			}

			CaseInsensitiveComparer caseInsensitiveComparer = new CaseInsensitiveComparer();
			foreach (Channel channel in allChannels)
			{
				foreach (Programme program in channel.getProgrammeList())
				{
					bool showProgram = !checkTitle && !checkDesc && !checkSubtitle && !matchTitle && !matchDesc && !matchSubtitle && !matchUniqueID;

					if (checkTitle) showProgram |= program.getTitle().ToUpper().IndexOf(searchFor) > -1;
					if (checkDesc) showProgram |= program.getDescription().ToUpper().IndexOf(searchFor) > -1;
					if (checkSubtitle) showProgram |= program.getSubTitle().ToUpper().IndexOf(searchFor) > -1;

					if (matchTitle) showProgram |= program.getTitle().ToUpper() == searchFor;
					if (matchDesc) showProgram |= program.getDescription().ToUpper() == searchFor;
					if (matchSubtitle) showProgram |= program.getSubTitle().ToUpper() == searchFor;
					if (matchUniqueID) showProgram |= program.getUniqueProgrammeIdentifier().ToUpper() == searchFor;

					double length = (program.getEndTime() - program.getStartTime()).TotalMinutes;
					showProgram &= (length >= minLength) && (length <= maxLength);

					if (showProgram && (selectedGenre != null) && (selectedGenre.Length > 0))
					{
						int idx = program.getGenreList().BinarySearch(selectedGenre, caseInsensitiveComparer);
						showProgram &= (idx > -1);
					}

					if (channels.Length > 0)
						showProgram &= Array.IndexOf(channels, channel.channelNumber.ToString()) > -1;

					if (showProgram) 
					{
						ScheduledRecording scheduledRecording = null;
						if (knownRecordings.Contains(program.getOID())) scheduledRecording = (ScheduledRecording)knownRecordings[program.getOID()];
						createOutputTable(program, channel, scheduledRecording, displayProgramme);
					}
				}
			}
			if (Recordings.Items.Count > 0)
			{
				System.Web.UI.WebControls.DropDownList FailedRecordings = new System.Web.UI.WebControls.DropDownList();
				string [] programmeInfo;
				knownRecordings = scheduleHelper.LoadKnownRecordings();
				for (int i = 0; i < Recordings.Items.Count; i++)
				{
					programmeInfo = Recordings.Items[i].Value.Split('@');
					if (!knownRecordings.Contains(System.Convert.ToInt32(programmeInfo[0])))
					{
						FailedRecordings.Items.Add(programmeInfo[1]);
					}
				}
				if (FailedRecordings.Items.Count > 0)
				{
					Recordings.Items.Clear();
					System.Web.UI.WebControls.ListItem [] failures = new ListItem[FailedRecordings.Items.Count];
					FailedRecordings.Items.CopyTo(failures,0);
					Recordings.Items.AddRange(failures);
					RecordMessage.Text = "Recordings That Failed to Schedule:";
					Recordings.Visible = true;
					RecordMessage.Visible = true;
				}
				else
				{
					Recordings.Items.Clear();
					Recordings.Visible = false;
					RecordMessage.Visible = false;
				}
			}

			displayProgramme.Dispose();
		}

		private void createOutputTable(Programme programme, Channel channel, ScheduledRecording scheduledRecording, ProgrammeDisplay displayProgramme)
		{

			// Program Description
			TableCell detailColDetails = new TableCell();
			displayProgramme.FillProgrammeDisplay(Server, detailColDetails, programme, scheduledRecording, true);

			TableRow detailRow = new TableRow();
			TableCell detailCol1 = new TableCell();
			if (scheduledRecording == null) {
				detailCol1.Text = "<input name=\"item" + programme.getOID().ToString() + "\" type=\"checkbox\" value=\"" + programme.getOID().ToString() + "\">";
			}
			detailCol1.CssClass = detailColDetails.CssClass;
			detailRow.Cells.Add(detailCol1);

			TableCell detailCol3 = new TableCell();
			detailCol3.Text =  channel.channelName;
			detailCol3.CssClass = detailColDetails.CssClass;
			detailRow.Cells.Add(detailCol3);

			detailRow.Cells.Add(detailColDetails);

			TableCell detailCol4 = new TableCell();
			detailCol4.Text = programme.getStartTime().ToLongDateString() + "<br><nobr>" + programme.getStartTime().ToShortTimeString() + " - " + programme.getEndTime().ToShortTimeString() + "</nobr>";
			detailCol4.CssClass = detailColDetails.CssClass;
			detailRow.Cells.Add(detailCol4);

			tableResults.Rows.Add(detailRow);
		}

		private void createHeaderRow() {
			TableRow headerRow = new TableRow();

			TableCell col1 = new TableCell();
			col1.CssClass = "header";
			col1.Text = "<nobr><input type=\"checkbox\" class=\"selectall\" onclick=\"SelectAllCheckboxes(this);\"><b>Select</b></nobr>";
			headerRow.Cells.Add(col1);

			TableCell col3 = new TableCell();
			col3.CssClass = "header";
			col3.Text = "Channel";
			headerRow.Cells.Add(col3);

			TableCell col3a = new TableCell();
			col3a.CssClass = "header";
			col3a.Text = "Show";
			headerRow.Cells.Add(col3a);

			TableCell col4 = new TableCell();
			col4.CssClass = "header";
			col4.Text = "Air Date";
			headerRow.Cells.Add(col4);

			tableResults.Rows.Add(headerRow);

		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.hlRecord.Click += new System.EventHandler(this.hlRecord_Click);
			this.Load += new System.EventHandler(this.Page_Load);
			this.PreRender += new System.EventHandler(this.SearchResults_PreRender);
		}
		#endregion
	}
}

