using System;
using System.Collections;

using GBPVR.Public;
using GBPVR.Backend.Common;

namespace gbweb
{
	/// <summary>
	/// Summary description for GBPVRSchedule.
	/// </summary>
	public class GBPVRSchedule : Schedule
	{
		public GBPVRSchedule()
		{
			//
			// TODO: Add constructor logic here
			//
			// RecordingFactor.getInstance() is not thread safe, so try and invoke it early where we're pretty sure we're single threaded
			LoadRecordingSchedule();
		}

		private IScheduleHelper scheduleHelper
		{
			get
			{
				return ScheduleHelper.getInstance();
			}
		}

		public override IList LoadRecordingSchedule()
		{
			return scheduleHelper.LoadRecordingSchedule();
		}

		public override Programme GetProgrammeByOID(int oid)
		{
			return scheduleHelper.GetProgrammeByOID(oid);
		}

		public override Channel GetChannelByOID(int oid)
		{
			// Can't use the interface for GetChannelByOID yet
			return ScheduleHelper.getInstance().GetChannelByOID(oid);
		}

		public override void CancelScheduledRecording(ScheduledRecording scheduledRecording)
		{
            scheduleHelper.CancelScheduledRecording(scheduledRecording);
		}

		public override IList GetListingsForTimePeriod(DateTime startTime, DateTime endTime)
		{
			return scheduleHelper.GetListingsForTimePeriod(startTime, endTime);
		}

		public override ScheduledRecording GetScheduledRecordingByOID(int oid)
		{
			// Can't use the interface for GetScheduledRecordingByOID yet
			IList myScheduledRecordings = scheduleHelper.LoadRecordingSchedule();					
			foreach (ScheduledRecording scheduledRecording in myScheduledRecordings)
			{
				if (scheduledRecording.getOID() == oid)
				{							
					return scheduledRecording;
				}
			}
			return null;
		}

		public override bool ScheduleOnce(Programme programme)
		{
			try
			{
				ScheduledRecording myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					programme.getStartTime(), 
					programme.getEndTime(), 
					programme, 
					ScheduledRecording.TYPE_RECORD_ONCE, 
					ScheduledRecording.GROUP_NONE, 
					null);
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleOnce(Programme programme, Quality quality)
		{
			try
			{
				ScheduledRecording myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					programme.getStartTime(), 
					programme.getEndTime(), 
					programme, 
					ScheduledRecording.TYPE_RECORD_ONCE, 
					ScheduledRecording.GROUP_NONE, 
					null, 
					(int)quality);
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleAnyTime(Programme programme)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = DateTime.MinValue;
				DateTime endTime = DateTime.MinValue;

				myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					startTime, 
					endTime, 
					null, 
					ScheduledRecording.TYPE_RECORD_SEASON, 
					ScheduledRecording.GROUP_ALLOCATE, 
					programme.getTitle());
			}
			catch
			{
				return false;
			}
			
			return true;
		}
		
		public override bool ScheduleAnyTime(Programme programme, Quality quality)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = DateTime.MinValue;
				DateTime endTime = DateTime.MinValue;

				myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					startTime, 
					endTime, 
					null, 
					ScheduledRecording.TYPE_RECORD_SEASON, 
					ScheduledRecording.GROUP_ALLOCATE, 
					programme.getTitle(), 
					(int)quality);
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleAnyTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = DateTime.MinValue;
				DateTime endTime = DateTime.MinValue;

				switch (dayType)
				{
					case DayType.SheduleAnyDay:
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON,
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings, 
							ReoccuringRecordingExtras.DAY_SUNDAY	|
							ReoccuringRecordingExtras.DAY_MONDAY	|
							ReoccuringRecordingExtras.DAY_TUESDAY	|
							ReoccuringRecordingExtras.DAY_WEDNESDAY	|
							ReoccuringRecordingExtras.DAY_THURSDAY	|
							ReoccuringRecordingExtras.DAY_FRIDAY	|
							ReoccuringRecordingExtras.DAY_SATURDAY);
						break;
					case DayType.SheduleThisDay:
						int day = (int)Math.Pow(2, (int)programme.getStartTime().DayOfWeek);
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON, 
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings,
							day);
						break;
					default:
						return false;
				}
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleAnyTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings, ArrayList days)
		{
			try
			{
				switch (dayType)
				{
					case DayType.SheduleSpecificDays:
						ScheduledRecording myRecording = new ScheduledRecording();
						DateTime startTime = DateTime.MinValue;
						DateTime endTime = DateTime.MinValue;

						Day day = 0;
						// Iterate through the ArrayList of days to create the daymask for the recording
						for (int i = 0; i < days.Count; i++)
						{
							day = day | (Day)days[i];
						}
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON, 
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings, 
							(int)day);
						break;
					default:
						return false;
				}
			}
			catch
			{
				return false;
			}

			return true;
		}

		public override bool ScheduleThisTime(Programme programme)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = programme.getStartTime();
				DateTime endTime = programme.getEndTime();

				myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					startTime, 
					endTime, 
					null, 
					ScheduledRecording.TYPE_RECORD_SEASON, 
					ScheduledRecording.GROUP_ALLOCATE, 
					programme.getTitle());
			}
			catch
			{
				return false;
			}
			
			return true;
		}
		
		public override bool ScheduleThisTime(Programme programme, Quality quality)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = programme.getStartTime();
				DateTime endTime = programme.getEndTime();

				myRecording = scheduleHelper.SheduleRecording(
					programme.getChannelOID(), 
					startTime, 
					endTime, 
					null, 
					ScheduledRecording.TYPE_RECORD_SEASON, 
					ScheduledRecording.GROUP_ALLOCATE, 
					programme.getTitle(), 
					(int)quality);
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleThisTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings)
		{
			try
			{
				ScheduledRecording myRecording = new ScheduledRecording();
				DateTime startTime = programme.getStartTime();
				DateTime endTime = programme.getEndTime();

				switch (dayType)
				{
					case DayType.SheduleAnyDay:
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON, 
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings	, 
							ReoccuringRecordingExtras.DAY_SUNDAY|
							ReoccuringRecordingExtras.DAY_MONDAY|
							ReoccuringRecordingExtras.DAY_TUESDAY|
							ReoccuringRecordingExtras.DAY_WEDNESDAY|
							ReoccuringRecordingExtras.DAY_THURSDAY|
							ReoccuringRecordingExtras.DAY_FRIDAY|
							ReoccuringRecordingExtras.DAY_SATURDAY);
						break;
					case DayType.SheduleThisDay:
						int day = (int)Math.Pow(2, (int)programme.getStartTime().DayOfWeek);
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON, 
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings, 
							day);
						break;
					default:
						return false;
				}
			}
			catch
			{
				return false;
			}
			
			return true;
		}

		public override bool ScheduleThisTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings, ArrayList days)
		{
			try
			{
				switch (dayType)
				{
					case DayType.SheduleSpecificDays:
						ScheduledRecording myRecording = new ScheduledRecording();
						DateTime startTime = programme.getStartTime();
						DateTime endTime = programme.getEndTime();

						Day day = 0;
						// Iterate through the ArrayList of days to create the daymask for the recording
						for (int i = 0; i < days.Count; i++)
						{
							day = day | (Day)days[i];
						}
						myRecording = scheduleHelper.SheduleRecording(programme.getChannelOID(),
							startTime, 
							endTime, 
							null, 
							ScheduledRecording.TYPE_RECORD_SEASON, 
							ScheduledRecording.GROUP_ALLOCATE,
							programme.getTitle(),
							(int)quality,
							keepnumRecordings, 
							(int)day);
						break;

					default:
						return false;
				}
			}
			catch
			{
				return false;
			}

			return true;
		}
	}
}
