using System;
using System.Collections;
using System.Collections.Specialized;

using GBPVR.Public;

namespace gbweb
{
	/// <summary>
	/// Summary description for Schedule.
	/// </summary>
	public abstract class Schedule
	{

		public abstract IList LoadRecordingSchedule();
		public abstract IList GetListingsForTimePeriod(DateTime startTime, DateTime endTime);
		public abstract Channel GetChannelByOID(int oid);
		public abstract Programme GetProgrammeByOID(int oid);
		public abstract ScheduledRecording GetScheduledRecordingByOID(int oid);
		public abstract void CancelScheduledRecording(ScheduledRecording scheduledRecording);
		
		public enum DayType
		{
			SheduleThisDay = 0,
			SheduleAnyDay = 1,
			SheduleSpecificDays = 2
		}

		public enum Quality
		{
			High = GBPVR.Public.ScheduledRecording.QUALITY_HIGH,
			Medium = GBPVR.Public.ScheduledRecording.QUALITY_MEDIUM,
			Low = GBPVR.Public.ScheduledRecording.QUALITY_LOW,
			Custom1 = GBPVR.Public.ScheduledRecording.QUALITY_CUSTOM1,
			Custom2 = GBPVR.Public.ScheduledRecording.QUALITY_CUSTOM2,
			LiveTV = GBPVR.Public.ScheduledRecording.QUALITY_LIVETV
		}

		[Flags]
		public enum Day
		{
			Sunday = GBPVR.Public.ReoccuringRecordingExtras.DAY_SUNDAY,
			Monday = GBPVR.Public.ReoccuringRecordingExtras.DAY_MONDAY,
			Tuesday = GBPVR.Public.ReoccuringRecordingExtras.DAY_TUESDAY,
			Wednesday = GBPVR.Public.ReoccuringRecordingExtras.DAY_WEDNESDAY,
			Thursday = GBPVR.Public.ReoccuringRecordingExtras.DAY_THURSDAY,
			Friday = GBPVR.Public.ReoccuringRecordingExtras.DAY_FRIDAY,
			Saturday = GBPVR.Public.ReoccuringRecordingExtras.DAY_SATURDAY
		}

		public const int Keep_All_Recordings = GBPVR.Public.ReoccuringRecordingExtras.KEEP_ALL_FILES;

		public abstract bool ScheduleOnce(Programme programme);
		public abstract bool ScheduleOnce(Programme programme, Quality quality);
		
		public abstract bool ScheduleAnyTime(Programme programme);
		public abstract bool ScheduleAnyTime(Programme programme, Quality quality);
		public abstract bool ScheduleAnyTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings);
		public abstract bool ScheduleAnyTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings, ArrayList days);
		
		public abstract bool ScheduleThisTime(Programme programme);
		public abstract bool ScheduleThisTime(Programme programme, Quality quality);
		public abstract bool ScheduleThisTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings);
		public abstract bool ScheduleThisTime(Programme programme, Quality quality, DayType dayType, int keepnumRecordings, ArrayList days);

		public IDictionary LoadKnownRecordings()
		{
			// build up a list of the recording we already know about
			IList scheduledRecordings = LoadRecordingSchedule();
			IDictionary knownRecordings = new HybridDictionary();
			foreach (ScheduledRecording scheduledRecording in scheduledRecordings)
			{
				if (scheduledRecording.getRecordingStatus() == ScheduledRecording.STATUS_DELETED)
					continue;

				if (scheduledRecording.getProgramme() != null)
				{
					knownRecordings[scheduledRecording.getProgramme().getOID()] = scheduledRecording;
				}
			}
			return knownRecordings;
		}

	}
}
